@@ -5,7 +5,7 @@ from pathlib import Path
 from types import SimpleNamespace
 from typing import Dict, Optional, Sequence, Tuple, Union, List
 
-from .solidpython import OpenSCADObject
+from .solidpython import IncludedOpenSCADObject, OpenSCADObject
 
 PathStr = Union[Path, str]
 
@@ -22,10 +22,6 @@ Indexes = Union[Sequence[int], Sequence[Sequence[int]]]
 ScadSize = Union[int, Sequence[float]]
 OpenSCADObjectPlus = Union[OpenSCADObject, Sequence[OpenSCADObject]]
 
-def _to_point2s(points:Points) -> List[P3]:
-    return list([(p[0], p[1]) for p in points])
-
-
 class polygon(OpenSCADObject):
     """
     Create a polygon with the specified points and paths.
@@ -40,15 +36,24 @@ class polygon(OpenSCADObject):
     assumed in order. (The 'pN' components of the *paths* vector are 0-indexed 
     references to the elements of the *points* vector.)
 
+    :param convexity: OpenSCAD's convexity... yadda yadda
+
     NOTE: OpenSCAD accepts only 2D points for `polygon()`. Convert any 3D points
     to 2D before compiling
     """
 
-    def __init__(self, points: Points, paths: Indexes = None) -> None:
-        if not paths:
-            paths = [list(range(len(points)))]
-        super().__init__('polygon',
-                         {'points': _to_point2s(points), 'paths': paths})
+    def __init__(self, points: Union[Points, IncludedOpenSCADObject], paths: Indexes = None, convexity: int = None) -> None:
+        # Force points to 2D if they're defined in Python, pass through if they're
+        # included OpenSCAD code
+        pts = points # type: ignore
+        if not isinstance(points, IncludedOpenSCADObject):
+            pts = list([(p[0], p[1]) for p in points]) # type: ignore
+
+        args = {'points':pts, 'convexity':convexity}
+        # If not supplied, OpenSCAD assumes all points in order for paths
+        if paths:
+            args['paths'] = paths # type: ignore
+        super().__init__('polygon', args)
 
 
 class circle(OpenSCADObject):
@@ -373,11 +378,14 @@ class color(OpenSCADObject):
     not specified.
 
     :param c: RGB color + alpha value.
-    :type c: sequence of 3 or 4 numbers between 0 and 1
+    :type c: sequence of 3 or 4 numbers between 0 and 1, OR 3-, 4-, 6-, or 8-digit RGB/A hex code, OR string color name as described at https://en.wikibooks.org/wiki/OpenSCAD_User_Manual/Transformations#color
+
+    :param alpha: Alpha value from 0 to 1
+    :type alpha: float 
     """
 
-    def __init__(self, c: Vec34) -> None:
-        super().__init__('color', {'c': c})
+    def __init__(self, c: Union[Vec34, str], alpha: float = 1.0) -> None:
+        super().__init__('color', {'c': c, 'alpha': alpha})
 
 
 class minkowski(OpenSCADObject):
@@ -747,50 +755,86 @@ def disable(openscad_obj: OpenSCADObject) -> OpenSCADObject:
 # ===========================
 # = IMPORTING OPENSCAD CODE =
 # ===========================
-def import_scad(scad_filepath: PathStr) -> Optional[SimpleNamespace]:
-    """
-    import_scad() is the namespaced, more Pythonic way to import OpenSCAD code.
-    Return a python namespace containing all imported SCAD modules
+def import_scad(scad_file_or_dir: PathStr) -> SimpleNamespace:
+    '''
+    Recursively look in current directory & OpenSCAD library directories for
+        OpenSCAD files. Create Python mappings for all OpenSCAD modules & functions
+    Return a namespace or raise ValueError if no scad files found
+    '''
+    scad = Path(scad_file_or_dir)
+    candidates: List[Path] = [scad]
+    if not scad.is_absolute():
+        candidates = [d/scad for d in _openscad_library_paths()]
 
-    If scad_filepath is a single .scad file, all modules will be imported,
-        e.g. 
-        motors = solid.import_scad('<PATH_TO/MCAD/motors.scad')
-        print(dir(motors)) # => [_stepper_motor_mount', 'stepper_motor_mount']
+    for candidate_path in candidates:
+        namespace = _import_scad(candidate_path)
+        if namespace is not None:
+            return namespace
+    raise ValueError(f'Could not find .scad files at or under {scad}. \nLocations searched were: {candidates}')
 
-    If scad_filepath is a directory, recursively import all scad files below
-        the directory and subdirectories within it.
-        e.g. 
-        mcad = solid.import_scad('<PATH_TO/MCAD')
-        dir(mcad) # => ['bearing', 'boxes', 'constants', 'curves',...]
-        dir(mcad.bearing) # => ['bearing', 'bearingDimensions', ...]
-    """
-    scad = Path(scad_filepath)
-
-    namespace: Optional[SimpleNamespace] = SimpleNamespace()
-    scad_found = False
-
-    if scad.is_file():
-        scad_found = True
-        use(scad.absolute().as_posix(), dest_namespace_dict=namespace.__dict__)
+def _import_scad(scad: Path) -> Optional[SimpleNamespace]:
+    '''
+    cases:
+        single scad file:
+            return a namespace populated with `use()`
+        directory
+            recurse into all subdirectories and *.scad files
+            return namespace if scad files are underneath, otherwise None
+        non-scad file:
+            return None            
+    '''
+    namespace: Optional[SimpleNamespace] = None
+    if scad.is_file() and scad.suffix == '.scad':
+        namespace = SimpleNamespace()
+        use(scad.absolute(), dest_namespace_dict=namespace.__dict__)
     elif scad.is_dir():
-        for f in scad.glob('*.scad'):
-            subspace = import_scad(f.absolute().as_posix())
-            setattr(namespace, f.stem, subspace)
-            scad_found = True
+        subspaces = [(f, _import_scad(f)) for f in scad.iterdir() if f.is_dir() or f.suffix == '.scad']
+        for f, subspace in subspaces:
+            if subspace:
+                if namespace is None:
+                    namespace = SimpleNamespace()
+                # Add a subspace to namespace named by the file/dir it represents
+                setattr(namespace, f.stem, subspace)
 
-        # recurse through subdirectories, adding namespaces only if they have
-        # valid scad code under them.
-        subdirs = list([d for d in scad.iterdir() if d.is_dir()])
-        for subd in subdirs:
-            subspace = import_scad(subd.absolute().as_posix())
-            if subspace is not None:
-                setattr(namespace, subd.stem, subspace)
-                scad_found = True
-
-    namespace = namespace if scad_found else None
     return namespace
+   
+def _openscad_library_paths() -> List[Path]:
+    """
+    Return system-dependent OpenSCAD library paths or paths defined in os.environ['OPENSCADPATH']
+    """
+    import platform
+    import os
+    import re
 
+    paths = [Path('.')]
 
+    user_path = os.environ.get('OPENSCADPATH')
+    if user_path:
+        for s in re.split(r'\s*[;:]\s*', user_path):
+            paths.append(Path(s))
+
+    default_paths = {
+        'Linux':   Path.home() / '.local/share/OpenSCAD/libraries',
+        'Darwin':  Path.home() / 'Documents/OpenSCAD/libraries',
+        'Windows': Path('My Documents\OpenSCAD\libraries')
+    }
+
+    paths.append(default_paths[platform.system()])
+    return paths
+
+def _find_library(library_name: PathStr) -> Path:
+    result = Path(library_name)
+
+    if not result.is_absolute():
+        paths = _openscad_library_paths()
+        for p in paths:
+            f = p / result
+            # print(f'Checking {f} -> {f.exists()}')
+            if f.exists():
+                result = f
+
+    return result
+ 
 # use() & include() mimic OpenSCAD's use/include mechanics.
 # -- use() makes methods in scad_file_path.scad available to be called.
 # --include() makes those methods available AND executes all code in
@@ -807,7 +851,7 @@ def use(scad_file_path: PathStr, use_not_include: bool = True, dest_namespace_di
     from .solidpython import new_openscad_class_str
     from .solidpython import calling_module
 
-    scad_file_path = Path(scad_file_path)
+    scad_file_path = _find_library(scad_file_path) 
 
     contents = None
     try:
@@ -837,6 +881,6 @@ def use(scad_file_path: PathStr, use_not_include: bool = True, dest_namespace_di
 
     return True
 
-
 def include(scad_file_path: PathStr) -> bool:
     return use(scad_file_path, use_not_include=False)
+
