@@ -17,6 +17,8 @@ from solid.test.ExpandedTestCase import DiffOutput
 
 scad_test_case_templates = [
     {'name': 'polygon', 'class': 'polygon' , 'kwargs': {'paths': [[0, 1, 2]]}, 'expected': '\n\npolygon(paths = [[0, 1, 2]], points = [[0, 0], [1, 0], [0, 1]]);', 'args': {'points': [[0, 0, 0], [1, 0, 0], [0, 1, 0]]}, },
+    {'name': 'polygon', 'class': 'polygon' , 'kwargs': {}, 'expected': '\n\npolygon(points = [[0, 0], [1, 0], [0, 1]]);', 'args': {'points': [[0, 0, 0], [1, 0, 0], [0, 1, 0]]}, },
+    {'name': 'polygon', 'class': 'polygon' , 'kwargs': {}, 'expected': '\n\npolygon(convexity = 3, points = [[0, 0], [1, 0], [0, 1]]);', 'args': {'points': [[0, 0, 0], [1, 0, 0], [0, 1, 0]], 'convexity': 3}, },
     {'name': 'circle', 'class': 'circle' , 'kwargs': {'segments': 12, 'r': 1}, 'expected': '\n\ncircle($fn = 12, r = 1);', 'args': {}, },
     {'name': 'circle_diam', 'class': 'circle' , 'kwargs': {'segments': 12, 'd': 1}, 'expected': '\n\ncircle($fn = 12, d = 1);', 'args': {}, },
     {'name': 'square', 'class': 'square' , 'kwargs': {'center': False, 'size': 1}, 'expected': '\n\nsquare(center = false, size = 1);', 'args': {}, },
@@ -35,7 +37,6 @@ scad_test_case_templates = [
     {'name': 'mirror', 'class': 'mirror' , 'kwargs': {}, 'expected': '\n\nmirror(v = [0, 0, 1]);', 'args': {'v': [0, 0, 1]}, },
     {'name': 'resize', 'class': 'resize' , 'kwargs': {'newsize': [5, 5, 5], 'auto': [True, True, False]}, 'expected': '\n\nresize(auto = [true, true, false], newsize = [5, 5, 5]);', 'args': {}, },
     {'name': 'multmatrix', 'class': 'multmatrix' , 'kwargs': {}, 'expected': '\n\nmultmatrix(m = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]]);', 'args': {'m': [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]]}, },
-    {'name': 'color', 'class': 'color' , 'kwargs': {}, 'expected': '\n\ncolor(c = [1, 0, 0]);', 'args': {'c': [1, 0, 0]}, },
     {'name': 'minkowski', 'class': 'minkowski' , 'kwargs': {}, 'expected': '\n\nminkowski();', 'args': {}, },
     {'name': 'offset', 'class': 'offset' , 'kwargs': {'r': 1}, 'expected': '\n\noffset(r = 1);', 'args': {}, },
     {'name': 'offset_segments', 'class': 'offset' , 'kwargs': {'r': 1, 'segments': 12}, 'expected': '\n\noffset($fn = 12, r = 1);', 'args': {}, },
@@ -113,7 +114,7 @@ class TestSolidPython(DiffOutput):
 
     def test_parse_scad_callables(self):
         test_str = """
-                    module hex (width=10, height=10,    
+                    module hex (width=10, height=10,
                                 flats= true, center=false){}
                     function righty (angle=90) = 1;
                     function lefty(avar) = 2;
@@ -143,17 +144,17 @@ class TestSolidPython(DiffOutput):
                     module var_with_arithmetic(var_with_arithmetic = 8 * 9 - 1 + 89 / 15){}
                     module var_with_parentheses(var_with_parentheses = 8 * ((9 - 1) + 89) / 15){}
                     module var_with_functions(var_with_functions = abs(min(chamferHeight2, 0)) */-+ 1){}
-                    module var_with_conditionnal_assignment(var_with_conditionnal_assignment = mytest ? 45 : yop){}
+                    module var_with_conditional_assignment(var_with_conditional_assignment = mytest ? 45 : yop){}
                    """
         expected = [
             {'name': 'hex', 'args': [], 'kwargs': ['width', 'height', 'flats', 'center']},
             {'name': 'righty', 'args': [], 'kwargs': ['angle']},
-            {'name': 'lefty', 'args': ['avar'], 'kwargs': []},
+            {'name': 'lefty', 'args': [], 'kwargs': ['avar']},
             {'name': 'more', 'args': [], 'kwargs': ['a']},
             {'name': 'pyramid', 'args': [], 'kwargs': ['side', 'height', 'square', 'centerHorizontal', 'centerVertical']},
             {'name': 'no_comments', 'args': [], 'kwargs': ['arg', 'other_arg', 'last_arg']},
             {'name': 'float_arg', 'args': [], 'kwargs': ['arg']},
-            {'name': 'arg_var', 'args': ['var5'], 'kwargs': []},
+            {'name': 'arg_var', 'args': [], 'kwargs': ['var5']},
             {'name': 'kwarg_var', 'args': [], 'kwargs': ['var2']},
             {'name': 'var_true', 'args': [], 'kwargs': ['var_true']},
             {'name': 'var_false', 'args': [], 'kwargs': ['var_false']},
@@ -172,7 +173,7 @@ class TestSolidPython(DiffOutput):
             {'name': 'var_with_arithmetic', 'args': [], 'kwargs': ['var_with_arithmetic']},
             {'name': 'var_with_parentheses', 'args': [], 'kwargs': ['var_with_parentheses']},
             {'name': 'var_with_functions', 'args': [], 'kwargs': ['var_with_functions']},
-            {'name': 'var_with_conditionnal_assignment', 'args': [], 'kwargs': ['var_with_conditionnal_assignment']}
+            {'name': 'var_with_conditional_assignment', 'args': [], 'kwargs': ['var_with_conditional_assignment']}
         ]
 
         from solid.solidpython import parse_scad_callables
@@ -182,7 +183,8 @@ class TestSolidPython(DiffOutput):
     def test_use(self):
         include_file = self.expand_scad_path("examples/scad_to_include.scad")
         use(include_file)
-        a = steps(3)
+
+        a = steps(3) # type: ignore
         actual = scad_render(a)
 
         abs_path = a._get_include_path(include_file)
@@ -205,6 +207,36 @@ class TestSolidPython(DiffOutput):
         expected = f"{header}\nuse <{abs_path}>\n\n\nsteps(howmany = 3);"
         self.assertEqual(expected, actual)
 
+        # Confirm that we can leave out even non-default arguments in OpenSCAD
+        a = mod.optional_nondefault_arg();
+        actual = scad_render(a)
+        expected = f'use <{abs_path}>\n\n\noptional_nondefault_arg();'
+        self.assertEqual(expected, actual);
+
+        # Make sure we throw ValueError on nonexistent imports
+        self.assertRaises(ValueError, import_scad, 'path/doesnt/exist.scad')
+
+        # Test that we recursively import directories correctly
+        examples = import_scad(include_file.parent)
+        self.assertTrue(hasattr(examples, 'scad_to_include'))
+        self.assertTrue(hasattr(examples.scad_to_include, 'steps'))
+
+        # TODO: we should test that:
+        # A) scad files in the designated OpenSCAD library directories
+        #       (path-dependent, see: solid.objects._openscad_library_paths())
+        #       are imported correctly. Not sure how to do this without writing
+        #       temp files to those directories. Seems like overkill for the moment
+
+    def test_imported_scad_arguments(self):
+        include_file = self.expand_scad_path("examples/scad_to_include.scad")
+        mod = import_scad(include_file)
+        points = mod.scad_points();
+        poly = polygon(points);
+        actual = scad_render(poly);
+        abs_path = points._get_include_path(include_file)
+        expected = f'use <{abs_path}>\n\n\npolygon(points = scad_points());'
+        self.assertEqual(expected, actual)
+
     def test_use_reserved_words(self):
         scad_str = '''module reserved_word_arg(or=3){\n\tcube(or);\n}\nmodule or(arg=3){\n\tcube(arg);\n}\n'''
 
@@ -215,12 +247,12 @@ class TestSolidPython(DiffOutput):
                 f.write(scad_str)
 
             use(path)
-            a = reserved_word_arg(or_=5)
+            a = reserved_word_arg(or_=5) # type: ignore
             actual = scad_render(a)
             expected = f"use <{path}>\n\n\nreserved_word_arg(or = 5);"
             self.assertEqual(expected, actual)
 
-            b = or_(arg=5)
+            b = or_(arg=5) # type: ignore
             actual = scad_render(b)
             expected = f"use <{path}>\n\n\nor(arg = 5);"
             self.assertEqual(expected, actual)
@@ -231,7 +263,7 @@ class TestSolidPython(DiffOutput):
         include_file = self.expand_scad_path("examples/scad_to_include.scad")
         self.assertIsNotNone(include_file, 'examples/scad_to_include.scad not found')
         include(include_file)
-        a = steps(3)
+        a = steps(3) # type: ignore
 
         actual = scad_render(a)
         abs_path = a._get_include_path(include_file)
@@ -272,6 +304,25 @@ class TestSolidPython(DiffOutput):
         actual = scad_render(root(a))
         self.assertEqual(expected, actual)
 
+    def test_color(self):
+        all_args = [
+            {'c': [1, 0, 0]},
+            {'c': [1, 0, 0], 'alpha': 0.5},
+            {'c': "#66F"},
+            {'c': "Teal", 'alpha': 0.5},
+        ]
+
+        expecteds = [
+            '\n\ncolor(alpha = 1.0000000000, c = [1, 0, 0]);',
+            '\n\ncolor(alpha = 0.5000000000, c = [1, 0, 0]);',
+            '\n\ncolor(alpha = 1.0000000000, c = "#66F");',
+            '\n\ncolor(alpha = 0.5000000000, c = "Teal");',
+        ]
+        for args, expected in zip(all_args, expecteds):
+            col = color(**args)
+            actual = scad_render(col)
+            self.assertEqual(expected, actual)
+
     def test_explicit_hole(self):
         a = cube(10, center=True) + hole()(cylinder(2, 20, center=True))
         expected = '\n\ndifference(){\n\tunion() {\n\t\tcube(center = true, size = 10);\n\t}\n\t/* Holes Below*/\n\tunion(){\n\t\tcylinder(center = true, h = 20, r = 2);\n\t} /* End Holes */ \n}'
@@ -384,7 +435,7 @@ class TestSolidPython(DiffOutput):
 
     def test_numpy_type(self):
         try:
-            import numpy
+            import numpy # type: ignore
             numpy_cube = cube(size=numpy.array([1, 2, 3]))
             expected = '\n\ncube(size = [1,2,3]);'
             actual = scad_render(numpy_cube)
